%   example1.m: Simple method of securing software, just checks for SmartDongle
%               presence.

%   www.smartdongle.com

%   MicroWorks, Inc.
%   2808 North Cole Road
%   Boise, ID 83704
%   (208)-375-1234

function example1()
    % Your personal keys go here.
    P1 = hex2uint64('EC6CC589AEFD1E75');
    P2 = hex2uint64('FCEC0A6A82747B3F');
    
    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Call the LedRed function from the USK library.
    calllib('USK','LedRed');
    
    % Create a pointer to a uint8 object.
    % The contents/size of this pointer are unimportant since no data will
    % actually be read into the object.
    buffer = libpointer('uint8Ptr', 0);
    
    % Call the Read function from the USK library to check for the presence
    % of a SmartDongle. No data is actually read since we are passing a
    % size of zero. The error code and alter buffer contents are returned
    % from this call.
    [error, buffer] = calllib('USK', 'Read', P1, P2, 0, buffer, 0);
    
    % Check the error code to see if the SmartDongle was successfully
    % accessed. Zero is a success all other numbers are an error.
    if(error ~= 0)
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\n%s\n\n',ErrMsg)
    else
        % A valid SmartDongle is present
        fprintf('\nValid SmartDongle Found\n\n')
    end
    
    % Call the LedRed function from the USK library.
    calllib('USK', 'LedGreen');
    
    % Unload the USK library
    unloadlibrary('USK');
end

