%    example3a.m:  Example that encrypts a user string and
%                  writes the encrypted string to SmartDongle.
%
%    www.smartdongle.com
%
%    MicroWorks, Inc.
%    2808 North Cole Road
%    Boise, ID 83704
%    (208)-375-1234

function example3a()
    % Your personal keys go here.
    P1 = hex2uint64('EC6CC589AEFD1E75');
    P2 = hex2uint64('FCEC0A6A82747B3F');
    
    % User space memory addres to write the data to on the SmartDongle. (0-32K)
    address = 0;
    
    % Data to be encrypted and then written to the SmartDongle. This can be
    % anything.
    WriteSecret = 'This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc';
    
    % Since all data to be written to the SmartDongle must be in byte array
    % format, convert the above string into a byte array.
    secretBytes = unicode2native(WriteSecret, 'US-ASCII');
    
    % This is the key that will be used to encrypt your data. It can be any
    % sixteen characters (128 bits) that you want.
    EncryptionKey = '7obdgEn2{TH>]SE|';
    % The encryption function requires that the key be in byte array form
    % so this function will convert the string to a byte array.
    keyBytes = unicode2native(EncryptionKey, 'US-ASCII');
    
    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Call LedRed from the USK library.
    calllib('USK','LedRed');
    
    % Create c-style pointers to the data to be encrypted and the
    % encryption key.
    secretPtr = libpointer('uint8Ptr', secretBytes);
    keyPtr = libpointer('uint8Ptr', keyBytes);
    
    % Create a c-style pointer to a uint8 object (byte array) to store the
    % data after encryption. Its size will usually be greater than that of
    % the data to be encrypted because the encryption process works on
    % blocks 16 bytes (128) and therefore will need more space if the size
    % of the data to be encrypted is not evenly divisible by 16.
    encryptedSecret = libpointer('uint8Ptr', uint8((zeros(1, 16 * (ceil(length(WriteSecret)/16))))));
    
    % Call the FileEncrypt function from the USK library to encrypt your
    % data. Pass a pointer to the byte array data to be encrypted, the size
    % of the data to be encrypted, a pointer to the byte array key to be
    % used for encryption, the size of the encryption block (always 128),
    % and a pointer to the byte array where the data will be stored after
    % encryption. Returns the original byte array data, the original byte
    % arra encryption key, and a byte array containing the encrypted data.
    [secretPtr, keyPtr, encryptedSecret] = calllib('USK','FileEncrypt', secretPtr, length(WriteSecret), keyPtr, 128, encryptedSecret);
    
    % Create a c-style pointer to the encrypted data.
    secretPtr = libpointer('uint8Ptr', encryptedSecret);
    
    % Call the Write function from the USK library to write the encrypted
    % data to the SmartDongle. (See write description in example2a)
    [error, secretPtr] = calllib('USK', 'Write', P1, P2, address, secretPtr, length(encryptedSecret));
    
    % An error code of zero indicates a success, all other numbers are
    % errors.
    if(error ~= 0)
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\nFailed: Could not write secret data to the SmartDongle\n%s\n\n',ErrMsg)
    else
        % Print success.
        fprintf('\nSuccess: Wrote secret data to the SmartDongle\n\n')
    end
    
    % Call LedGreen from the USK library.
    calllib('USK', 'LedGreen');
    
    % Unload the USK library.
    unloadlibrary('USK');
end

