%   example4.m

%   Retrieves serial number of a single SmartDongle on the host.

%   www.smartdongle.com

%   MicroWorks, Inc.
%   2808 North Cole Road
%   Boise, ID 83704
%   (208)-375-1234

function example4()

    % Load the USK library for SmartDongle functions.
    loadlibrary('USK', 'usk.h');
    
    % Create a pointer for a c-style string with 12 characters to hold the
    % serial number of the SmartDongle.
    sn = libpointer('cstring', blanks(12));
    
    % Call GetSerialNumber from the USK library passing the sn pointer. An
    % error code and the serial number will be returned from this call.
    [error, sn] = calllib('USK','GetSerialNumber', sn);
    
    % A error code of zero is a success, all other numbers are errors.
    if(error == 0)
        % Print the serial number on success.
        fprintf('\nSerial Number: %s\n\n',sn)
    else
        % Create a pointer to a c-style string with 128 characters to allow
        % space for the max error message length.
        ErrMsg = libpointer('cstring', blanks(128));
        
        % Call GetErrorString from the USK library passing in the c-style
        % string ErrMsg and the error code from the previous Read call. The
        % error message will be returned into ErrMsg.
        [ErrMsg] = calllib('USK', 'GetErrorString', ErrMsg, error);
        
        % Print the error message.
        fprintf('\n%s\n\n',ErrMsg)
    end
    
    % Unload the USK library.
    unloadlibrary('USK');
end

